// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/SolverEngine.hpp>

#include <fstream>
#include <iostream>

SolverEngine::SolverEngine(const std::vector<std::string>& inputPaths)
    : inputPaths_{ inputPaths }
{
}

void SolverEngine::run(Solver& solver)
{
    std::cout << "\n--- Day " << solver.getPuzzleDay() << ": " << solver.getPuzzleName() << " ---\n";

    auto fullFilePath = tryGetValidFullInputFilePath(solver.getInputFileName());
    if (fullFilePath != "")
    {
        std::string line;
        std::ifstream inputFile{ fullFilePath };
        while (std::getline(inputFile, line))
        {
            solver.processDataLine(line);
        }
        inputFile.close();

        solver.finish();

        std::cout << "Part 1: " << solver.getResultPart1() << "\nPart 2: " << solver.getResultPart2() << std::endl;
    }
}

std::filesystem::path SolverEngine::tryGetValidFullInputFilePath(const std::string& inputFileName)
{
    for (auto path : inputPaths_)
    {
        std::filesystem::path fullFilePath = path;
        fullFilePath /= inputFileName;
        if (std::filesystem::exists(fullFilePath))
        {
            return fullFilePath;
        }
    }

    std::cout << "Cannot find puzzle input file '";
    for (size_t i = 0; i < inputPaths_.size(); i++)
    {
        std::filesystem::path fullFilePath = inputPaths_[i];
        fullFilePath /= inputFileName;
        std::cout << std::filesystem::absolute(fullFilePath).string();
        if (i + 2 < inputPaths_.size())
        {
            std::cout << "', '";
        }
        else if (i + 1 < inputPaths_.size())
        {
            std::cout << "', or '";
        }
    }
    std::cout << "'. Please download the file content from https://adventofcode.com/2024/\n";

    return "";
}
