// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/WarehouseWoes.hpp>

WarehouseWoes::WarehouseWoes(const int inputFileNameSuffix)
    : LinesSolver{ inputFileNameSuffix }, isProcessingMap_{ true }, robotPosition_{}
{
}

const std::string WarehouseWoes::getPuzzleName() const
{
    return "Warehouse Woes";
}

const int WarehouseWoes::getPuzzleDay() const
{
    return 15;
}

void WarehouseWoes::processDataLine(const std::string& line)
{
    if (!line.empty())
    {
        if (isProcessingMap_)
        {
            auto hit = line.find(getRobotChar());
            if (hit != std::string::npos)
            {
                robotPosition_ = { static_cast<int>(hit), static_cast<int>(lines.size()) };
            }
            LinesSolver::processDataLine(line);
        }
        else
        {
            processDirections(line);
        }
    }
    else
    {
        isProcessingMap_ = false;
    }
}

void WarehouseWoes::finish()
{
    for (size_t j = 0; j < lines.size(); j++)
    {
        for (size_t i = 0; i < lines[j].size(); i++)
        {
            if (lines[j][i] == getBoxChar())
            {
                part1 += calcGpsCoordinate(i, j);
            }
        }
    }
}

constexpr char WarehouseWoes::getRobotChar()
{
    return '@';
}

constexpr char WarehouseWoes::getBoxChar()
{
    return 'O';
}

constexpr char WarehouseWoes::getWallChar()
{
    return '#';
}

constexpr char WarehouseWoes::getEmptyChar()
{
    return '.';
}

void WarehouseWoes::processDirections(const std::string& line)
{
    for (const char c : line)
    {
        const Point2 direction{ Point2::getCardinalDirection(c) };
        const Point2 next{ robotPosition_ + direction };
        Point2 push{ next };
        while (getCharAt(push) == getBoxChar())
        {
            push += direction;
        }
        if (getCharAt(push) != getWallChar())
        {
            setCharAt(push, getBoxChar());
            setCharAt(next, getEmptyChar());
            robotPosition_ = next;
        }
    }
}

int WarehouseWoes::calcGpsCoordinate(const size_t x, const size_t y)
{
    return x + 100 * y;
}
